
# q-expansion of newform 320.2.n.c, downloaded from the LMFDB on 15 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 320
weight = 2
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[191, [-1, 0]], [261, [1, 0]], [257, [0, 1]]]
aps_data = [[0, 0], [-1, -1], [-1, 2], [1, -1], [0, -6], [1, -1], [1, 1], [4, 0], [-5, -5], [0, -8], [0, -2], [5, 5], [6, 0], [3, 3], [-7, 7], [1, -1], [4, 0], [-2, 0], [-7, 7], [0, 6], [9, -9], [8, 0], [-5, -5], [0, 0], [-3, -3], [-6, 0], [3, 3], [-3, 3], [0, -4], [-3, 3], [5, -5], [0, 2], [13, 13], [12, 0], [0, 12], [0, -18], [-3, -3], [-1, -1], [5, -5], [-7, 7], [-12, 0], [10, 0], [0, -10], [1, -1], [1, 1], [0, 0], [0, 10], [19, 19], [13, -13], [0, -16], [13, -13], [16, 0], [14, 0], [0, 18], [-11, -11], [-9, -9], [0, 4], [0, 22], [9, 9], [-10, 0], [15, 15], [17, -17], [-7, 7], [0, 22], [-15, 15], [25, 25], [0, 18], [1, 1], [9, -9], [0, -32], [-15, 15], [16, 0], [13, -13], [21, -21], [4, 0], [-13, -13], [0, 12], [-19, -19], [10, 0], [0, 20], [12, 0], [-2, 0], [0, 38], [5, -5], [-32, 0], [-17, -17], [0, 4], [-15, -15], [2, 0], [-17, -17], [5, -5], [24, 0], [-15, 15], [0, 2], [-20, 0], [11, 11], [0, 0], [18, 0], [-25, -25], [-30, 0], [-3, 3], [-15, -15], [15, 15], [0, -20], [0, 26], [-3, -3], [25, -25], [-23, 23], [-16, 0], [-26, 0], [-15, 15], [-3, 3], [-11, -11], [-28, 0], [0, -10], [-18, 0], [11, 11], [-31, 31], [5, -5], [44, 0], [30, 0], [29, -29], [-15, -15], [-13, -13], [0, 10], [-2, 0], [0, 24], [16, 0], [-15, 15], [9, -9], [-28, 0], [-9, -9], [0, -26], [1, 1], [50, 0], [0, -8], [-35, 35], [9, -9], [17, 17], [0, 0], [0, -30], [22, 0], [3, 3], [-7, 7], [0, 36], [24, 0], [-27, 27], [5, 5], [20, 0], [-25, -25], [-27, -27], [-34, 0], [-17, -17], [-7, 7], [-27, 27], [0, -50], [0, 0], [0, 12], [-3, -3], [2, 0], [41, -41], [9, -9], [37, -37], [0, 26], [-27, -27], [-41, -41], [0, -34], [-27, -27]]
